# -*- coding:utf-8 -*-
import os, selinux, subprocess
from security_module import SecurityModule

from utils import check_package, create_file, create_symbolic
from log import log_debug, log_error, LogManager

SECONFIG="/etc/selinux/config"
RELABEL_FILE="/.autorelabel"


class SELinuxModule(SecurityModule):
	status_desc = {
		"0": "关闭",
		"2": "启用(Enforcing)",
		"4": "启用(Permissive)"
	}
	def __init__(self):
		SecurityModule.__init__(self, "selinux")
		self.policy_name=None

	@staticmethod
	def getstatus():
		if selinux.is_selinux_enabled():
			enforce = selinux.security_getenforce()
			if enforce:
				return 2
			else:
				return 4	
		return 0

	def check_packages(self, modules):
		result, pkg_list=SecurityModule.check_packages(self, modules)

		if "admin3" in modules:
			pkg_name = "selinux-policy-ukmcs"
			self.policy_name = "ukmcs"
			log_debug("3Admin enabled, use '%s' as default policy"%self.policy_name)
		else:
			pkg_name = "selinux-policy-targeted"
			self.policy_name = "targeted"
			log_debug("3Admin disabled, use '%s' as default policy"%self.policy_name)

		if not check_package(pkg_name):
			if not pkg_list:
				pkg_list=[]
			pkg_list.append(pkg_name)
			return (False, pkg_list)

		return (result, pkg_list)

	def do_enable(self):
		try:
			expr="/^SELINUX=/cSELINUX=enforcing"
			log_debug("Updating SELinux configuration ...\n")
			subprocess.check_call(["/bin/sed", "-i", expr, SECONFIG], stderr=LogManager.getfd())
		except Exception as e:
			error="切换SELinux状态时出错: %s"%e.message
			log_error(error)
			return (False, error)

		try:
			expr="/^SELINUXTYPE=/cSELINUXTYPE=%s"%self.policy_name
			log_debug("Updating SELinux configuration ...\n")
			subprocess.check_call(["/bin/sed", "-i", expr, SECONFIG], stderr=LogManager.getfd())
			#log_debug("Regenerating contexts for homedirs ...\n")
			#subprocess.check_call(["genhomedircon"], stderr=LogManager.getfd())
		except Exception as e:
			error="切换SELinux策略时出错: %s"%e.message
			log_error(error)
			return (False, error)

		try:
			result, error = create_file(RELABEL_FILE)
			if not result:
				raise Exception(error)
		except Exception as e:
			error="无法创建标记文件 '%s': %s"%(RELABEL_FILE, e.message)
			log_error(error)
			return (False, error)

		return (True, None)

	def do_disable(self):
		try:
			if os.path.exists(RELABEL_FILE):
				os.unlink(RELABEL_FILE)
		except Exception as e:
			error="无法删除标记文件 '%s': %s"%(RELABEL_FILE, e.message)
			log_error("%s: %s"%(error, str(e)))
			return (False, error)

		try:
			if os.path.exists(SECONFIG):
				expr="/^SELINUX=/cSELINUX=disabled"
				log_debug("Updating SELinux configuration ...\n")
				subprocess.check_call(["/bin/sed", "-i", expr, SECONFIG], stderr=LogManager.getfd())
		except Exception as e:
			error="切换SELinux状态时出错: %s"%e.message
			log_error(error)
			return (False, error)

		return (True, None)
