# -*- coding:utf-8 -*-
import subprocess, getpass, platform
import sys, pam, os

from log import log_error, log_debug, LogManager

KYSEC_U_SERIAL=1

def print_err(string):
	'''
	将给定字符串打印到错误输出
	'''
	sys.stderr.write(string)

def package_info(pkg_name):
	'''
	获取给定Debian/Rpm软件包的安装状态和版本信息

	参数:
	  pkg_name: 软件包名称
	
	返回值:
	  Debian软件包返回值格式为 (安装状态, 安装版本)，若包未安装，返回值为(None, None)
	  Rpm软件包返回值格式为 True/False
	'''
	if KYSEC_U_SERIAL:
		try:
			args = ["dpkg-query","-W", "-f", "${db:Status-Status} ${Version}", pkg_name]
			output = subprocess.check_output(args, universal_newlines=True)
			tokens = output.split(" ", 2)
			if len(tokens) < 2:
				return (None, None)
			return (tokens[0], tokens[1])
		except subprocess.CalledProcessError as e:
			return (None, None)
		except Exception as e:
			return (None, None)
	else:
		try:
			args = "rpm -qa | grep %s"%pkg_name
			output = subprocess.check_output(args, shell=True, universal_newlines=True)
			if len(output) > 0:
				return True
			else:
				return False
		except subprocess.CalledProcessError as e:
			return False
		except Exception as e:
			return False

def check_package(pkg_name, version_check=False):
	'''
	检查当前系统中给定的软件包是否已安装或版本满足要求

	参数:
	  pkg_name: 软件包名称
	  version_check: 是否进行版本检查, True表示需要，False表示不需要

	返回值：
	  检查成功返回True，否则返回False 
	'''

	if KYSEC_U_SERIAL:
		status, version=package_info(pkg_name)
		if status == "installed":
			if version_check:
				return version.find("sec")>0
			return True
		else:
			return False
	else:
		return package_info(pkg_name)

def check_kconfig(config):
	'''
	检查系统当前内核编译时是否启用了给定的配置

	参数：
	  config: 内核配置名称，例如 “CONFIG_SECURITY"
	
	返回值：
	  启用返回True，否则返回False
	'''
	arch = platform.uname()[4].lower()
	version = platform.uname()[2]

	if arch == "aarch64"  and os.path.exists("/proc/config.gz"):
		args = ["zcat","/proc/config.gz"]
	elif os.path.exists("/boot/config-"+version):
		args = ["cat", "/boot/config-"+version]
	else:
		return (False, "内核配置文件未找到")

	try:
		child1 = subprocess.Popen(args, stdout=subprocess.PIPE, stderr=LogManager.getfd(), bufsize=1, universal_newlines=True)
		while True:
			line=child1.stdout.readline()
			if not len(line):
				break

			if line.find("%s=y"%config) >= 0:
				child1.terminate()
				return (True, None)
	except Exception as e:
		return (False, str(e))

	return (False, None)

def check_ksymbol(symbol):
	SYMBOL_PATH="/proc/kallsyms"
	try:
		found = False
		fd = open(SYMBOL_PATH, "r")
		while True:
			line = fd.readline()
			if not len(line):
				break
			s_addr, s_type, s_name = line.replace("\n", "").split(" ")
			if s_name == symbol:
				found = True
				break
	except Exception as e:
		retval = (False, str(e))
	else:
		retval = (found, None)
	finally:
		fd.close()
		return retval


def check_user(user_name, user_id, user_homedir, groups):
	'''
	检查当前系统中是否存在给定的用户满足指定的条件(拥有对应UID和主目录，属于给定的用户组)

	参数:
	  user_name: 要检查的用户名
	  user_id  : 用户user_name应对应的uid
	  user_homedir: 用户user_name的主目录路径
	  groups   : 用户user_name所在的用户组列表

	返回值:
	  成功返回(True, None),失败返回(False, REASON)，其中REASON为错误信息
	'''
	try:
		args = ["/usr/bin/getent", "passwd", str(user_id)]
		output = subprocess.check_output(args)
	except subprocess.CalledProcessError as e:
		if (e.returncode == 2):
			#return (False, "User with ID %s not found"%(user_id))
			print("无 '%s' 用户, 创建中..."%(user_name))
			rc = os.system("useradd -m -d /home/{name} -g {uid} -u {uid} -s /bin/bash {name}".format(name=user_name, uid=user_id))
			if rc != 0:
				return (False, "创建 '%s' 用户失败"%(user_name))
			if user_name == "auditadm":
				os.system("usermod -a -G adm auditadm")
				if KYSEC_U_SERIAL:
					os.system("usermod -a -G syslog auditadm")
			return (True, None)
		else:
			return (False, str(e)) 
	except Exception as e:
		return (False, str(e))
	else:
		name,placeholder,uid,gid,placeholder,homedir,shell=output.decode().replace("\n", "").split(":")

		if (user_name != name):
			return (False, "User with ID %s is not '%s'!"%(user_id, user_name))
		if (user_homedir and homedir != user_homedir):
			return (False, "User %s exists, but homedir is not '%s'!"%(user_name, user_homedir))

	try:
		output = subprocess.check_output(["/usr/bin/groups", user_name])
	except subprocess.CalledProcessError as e:
		if (e.returncode == 2):
			#return (False, "User %s not found"%(user_name))
			print("无 '%s' 用户, 创建中..."%user_name)
			rc = os.system("useradd -m -d /home/{name} -g {uid} -u {uid} -s /bin/bash {name}".format(name=user_name, uid=user_id))
			if rc != 0:
				return (False, "创建 '%s' 用户失败"%(user_name))
			if user_name == "auditadm":
				os.system("usermod -a -G adm auditadm")
				if KYSEC_U_SERIAL:
					os.system("usermod -a -G syslog auditadm")
			return (True, None)
		else:
			return (False, str(e))
	except Exception as e:
		return (False, str(e))
	else:
		output=output.decode().replace("%s : "%user_name, "").replace("\n", "")
		groups_list = output.split(" ")

		for group in groups:
			if group not in groups_list:
				return (False, "User %s not in group '%s'"%(user_name, group))

		return (True, None)


def check_group(group_name, group_id=None):
	'''
	检查系统中是否存在给定的用户组

	参数：
	  group_name: 待检查的用户组名
	  group_id  : 用户组对应的gid，如果为None，函数将不会对group_name对应的gid进行检查

	返回值:
	  成功返回 (True, None)，失败返回(False, REASON)，REASON为错误信息
	'''
	try:
		output = subprocess.check_output(["/usr/bin/getent", "group", group_name])
	except subprocess.CalledProcessError as e:
		if (e.returncode == 2):
			#return (False, "Group %s not found"%group_name)
			print("无 '%s' 用户组, 创建中..."%group_name)
			rc = os.system("groupadd -g {gid} {name}".format(gid=group_id, name=group_name))
			if rc != 0:
				return (False, "创建 '%s' 用户组失败"%group_name)
			else:
				return (True, None)
		else:
			return (False, str(e))
	except Exception as e:
		return (False, str(e))
	else:
		name,placeholder,gid,placeholder=output.decode().split(":")

		if (group_id and gid != str(group_id)):
			return (False, "Group %s exists, but ID is not %d"%(name, group_id))

		return (True, None)


def check_login(user, passwd):
	'''
	验证给定用户是否可以使用给定密码进行登录, 使用login服务的pam配置进行检查

	参数:
	  user: 用户名
	  passwd: 密码

	返回值:
	  验证成功返回 True，验证失败返回False
	'''
	if not user or not passwd:
		return False

	p=pam.pam()
	result=p.authenticate(user, passwd, "login")

	if result:
		return True
	else:
		return False


def enable_user(user, auto_reset = False):
	'''
	激活给定的用户账户，并重置其密码

	成功返回(True, None)，失败返回(False, REASON)，其中REASON为错误信息

	参数:
	  user: 需要激活的用户名
	  passwd: 密码明文，若设置为None，用户密码不会改变
	'''
	try:
		if not auto_reset:
			print("%s 用户密码需要手动设置"%user)
			while True:
				try:
					passwd_1=getpass.unix_getpass("请输入新密码: ")
					passwd_2=getpass.unix_getpass("请再次输入新密码进行确认: ")
				except EOFError as e:
					continue
				except Exception as e:
					return (False, "设置密码时出错: %s"%str(e))

				if passwd_1 != passwd_2:
					print("两次密码不匹配")
				else:
					break
		else:
			passwd_1 = "123123"

		up_input="{u}:{p}".format(u=user, p=passwd_1)
		child = subprocess.Popen(["/usr/sbin/chpasswd"], stdin=subprocess.PIPE, stderr=subprocess.PIPE, universal_newlines=True)
		out, error = child.communicate(up_input)
		if child.returncode != 0:
			#return (False, error.decode("utf-8"))
			return (False, str(error))

		if auto_reset:
			try:
				subprocess.check_call(["/usr/bin/passwd", "-e", user])
			#except CalledProcessError as e:
			except Exception as e:
				return (False, "设置账户过期时间时出错")
		return (True, None)

	except Exception as e:
		return (False, str(e))


def disable_user(user, resetpw=False):
	'''
	禁用给定的用户账户
	
	禁用是通过锁定账户并将其账户设置为过期来实现的
	操作成功返回(True, None)，失败返回(False, REASON)，其中REASON为错误信息

	参数:
	  user: 需要禁用的用户
	  resetpw: 是否要在账户锁定后清除当前密码，若不清除，账户解锁后可以使用之前的unix密码登录。True表示清除，False表示不清除
	'''
	try:
		child = subprocess.Popen(["/usr/sbin/chpasswd", "-e"], stdin=subprocess.PIPE, stderr=subprocess.PIPE, universal_newlines=True)
		out, error = child.communicate("%s:*"%user)
		if child.returncode != 0:
			#return (False, error.decode("utf-8"))
			return (False, str(error))

		return (True, None)

	except Exception as e:
		return (False, str(e))


def create_file(file_path):
	'''
	在文件系统上创建给定的文件

	参数：
	  file_path: 文件路径	

	返回值：
	  创建成功返回(True, None)，创建失败返回(False, REASON)，REASON为失败信息
	'''
	if os.path.exists(file_path):
		return (True, None)

	try:
		fd = open(file_path, "w")
	except Exception as e:
		return (False, str(e))
	else:
		fd.close()
		return (True, None)

def create_symbolic(src_file, dst_file):
	if not os.path.exists(src_file):
		raise Exception("文件 '%s' 不存在"%src_file)

	if os.path.lexists(dst_file):
		os.unlink(dst_file)

	os.symlink(src_file, dst_file)


def unlink_symbolic(dst_file):
	if os.path.lexists(dst_file):
		os.unlink(dst_file)

def write_config(config_file, mode):
	try:
		config = open(config_file, "w")
		config.write("mode=%s\n"%mode)
	except Exception as e:
		return (False, str(e))
	else:
		config.close()
		return (True, None)


def read_config(config_file):
	try:
		config = open(config_file, "r")
		line = config.readline()
		if line.find("mode="):
			raise Exception("invalid format")
		mode = line.replace("mode=", "").replace("\n", "")
	except Exception as e:
		return (False, str(e))
	else:
		config.close()
		return (True, mode)


def write_custom_mode(custom_file, modules):
	try:
		custom = open(custom_file, "w")
		for module_name in modules:
			custom.write("%s\n"%module_name)
	except Exception as e:
		return (False, str(e))
	else:
		custom.close()
		return (True, None)


def read_custom_mode(custom_file):
	try:
		modules = []
		custom = open(custom_file, "r")
		while True:
			line = custom.readline()
			if not len(line):
				break
			module_name = line.replace("\n", "")
			modules.append(module_name);
	except Exception as e:
		return (False, [str(e)])
	else:
		custom.close()
		return (True, modules)


if __name__ == "__main__":
	print(check_ksymbol("kysec_init"))
	print(check_ksymbol("selinux_init"))
	if KYSEC_U_SERIAL:
		print(check_ksymbol("apparmor_init"))
